/* mon.c   command line interpreter for sim

	10 November 2001
	20 May 2003
	modify for xs1       14 Sept 2005
	for cp8299x			 18 Dec 2005
	P. Chongstitvatana

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MXBUF  80
#define MXOBUF 2000
#define sep  "\t\n "

extern int pc, op, ac, sp, Cf, Zf;
extern char M[];
extern FILE *fi;

int pc2, bp;
char ibuf[MXBUF], *cp = NULL;		// input buffer
int disp = 0;						// display control
int limit = 20000;
int runflag;

char *getname(int op);
void run(void);

// get one token from input
char *tok(void){
	if( cp != NULL ) cp = strtok(NULL,sep);
	while ( cp == NULL ){
		printf(">");
		cp = fgets(ibuf,MXBUF,fi);
		if( cp != NULL ) cp = strtok(ibuf, sep);
	}
	cp = strupr(cp);   /* convert to uppercase */
// printf("%s ",cp);
	return cp;
}
// convert hex to integer
int htoi(char *s){
	char c;
	int v, h;
	h = 0;
	v = 0;
	while( (c = *s) != 0 ){
		if(c >= '0' && c <= '9') v = (int)(c-'0');
		else if(c >= 'A' && c <= 'F') v = (int)(c-'A'+10);
		h = h * 16 + v;
		s++;
	}
	return h;
}

void show(void){
	printf("pc:%04X %3s  A:%02X  SP:%04X  C%d Z%d\n",
		pc2,getname(op),ac&255,sp&0x0ffff,Cf,Zf);
}

void trace(void){
	while((pc != bp) && runflag) {	// not breakpoint
		if( (limit--) < 0 ) break;
		run();
		if(disp) show();
	}
}

// dump memory for n bytes
void dumpMem(int ads, int n){
	int i,k;
	k = 0;
	printf("%04X : ",ads);
	for(i=ads; i<ads+n; i++){
		printf("%02X ",(unsigned char)M[i]);
		k++;
		if( k%10 == 0 ) printf("\n%04X : ",i+1);
	}
	printf("\n");
}

void help(void){
	printf("g [go], t [single step], d ads n [dump DS]\n");
	printf("b ads [set breakpoint], c [clear breakpoint]\n");
    printf("z 1/0 [on/off display], q [quit], h [help]\n");
}

void interp(int ref){
	char *w, *arg, *v;
    int flag;
	flag = 1;
	pc = ref;
	runflag = 1;
	fi = stdin;
	cp = NULL;
	bp = -1;
	while(flag){
		w = tok();
		switch( w[0] ){
		case 'G': trace(); break;
		case 'T': run(); show(); break;
		case 'B': bp = htoi(tok());	break;
		case 'C': bp = -1; break;
		case 'D':
			arg = tok();
			v = tok();
			dumpMem(htoi(arg),htoi(v));
			break;
		case 'Z': disp = htoi(tok()); break;
		case 'Q': flag = 0; break;
		case 'H': help(); break;
		default: printf("unknow command\n");
		}
	}
}
